/*
*************************************************************************************
* Disc:  					A basic program for Q's 8x6 PLC Project														*
*									Includes all I/O init functions, some basicl UART com functions		*
*									& EEPROM functions functionality																	*
*																																										*
* Version:				Alpha																															*
* Date:						1 Dec 2009																												*																																					*
* Demo Project								*
*************************************************************************************
*/


// *** INCLUDES
#include <avr/io.h> 
#include <stdint.h>

// *** USART DEFINES
#define USART_BAUDRATE 9600
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1) 



// **********************************************************************************
// *** IO INIT

void IOSetupInit(void){
	// PB0=IN 8,	PB1=IN 1	PB2=IN 2	PB3=PWM,	PB4=IN 3		PB5=IN 4
	PORTB = 0;
	DDRB = (1<<3);

	// PC0=A IN,		PC1=INPUT 5		PC2=INPUT 6
	// PC3 = INPUT 7
	PORTC = 0;
	DDRC = 0;

	// PD2=0UT 1	PD3=OUT 2,	PD4=OUT 3	PD5=OUT 4	PD6=OUT 5		PD7=OUT 6
	PORTD = 0;
	DDRD = (1<<DDD2)|(1<<DDD3)|(1<<DDD4)|(1<<DDD5)|(1<<DDD6)|(1<<DDD7);
}



// **********************************************************************************
// *** TIMER INIT
void TimerInit(void){
   	TCCR1B |= (1 << WGM12); 				// Configures timer 1 for CTC mode
   	TIMSK1 |= (1 << OCIE1A); 				// Enable CTC interrupt
   	OCR1A = (31249); 						// Set CTC compaire value to 1Hz at 8Mhz

   	TCCR1B |= (1 << CS10) | (1 << CS11);	// Start timer with 256 prescale 	
}


// UNTESTED
// **********************************************************************************
// *** PWM INIT
void PWMInit(void){
	TCCR2A |= (1 << WGM20) | (1 << WGM21);	// Set Fast PWM mode
	TCCR2A |= (1 << COM2A0);				// TOGGLE ON OC2A MATCH

	OCR2A = 0x30;
	TCCR2B |= (1 << CS20);					// START WITH NO PRESCALE
	
}


// UNTESTED
// **********************************************************************************
// *** TIMER FOR RANDOM NUMBER GENERATOR
void RandomTimerInit(void){
	TCCR0A |= (1 << WGM01);					// CTC MODE


}



// **********************************************************************************
// *** USART INIT
void UartInit(void){
   	UCSR0B |= (1 << RXEN0) | (1 << TXEN0);   				// Turn on the TX and RX circuitry 
  	UCSR0C |= /*(1 << URSEL0) |*/ (1 << UCSZ00) | (1 << UCSZ01);// Use 8-bit character sizes at Asynchronous 
  	UBRR0L = BAUD_PRESCALE; 								// Load lower 8-bits of the baud rate value into the low byte of the UBRR register 
  	UBRR0H = (BAUD_PRESCALE >> 8); 						// Load upper 8-bits of the baud rate value into the high byte of the UBRR register 
  	UCSR0B |= (1 << RXCIE0); 								// Enable the USART Receive Complete interrupt (USART_RXC) 
}





