/*
*************************************************************************************
* Disc:  					A basic program for Q's 8x6 PLC Project														*
*									Includes all I/O init functions, some basicl UART com functions		*
*									& EEPROM functions functionality																	*
*																																										*
* Version:				Alpha																															*
* Date:						1 Dec 2009																												*																																	*
* Demo Project								*
*************************************************************************************
*/



// *** INCLUDES
#include <avr/io.h> 
#include <avr/interrupt.h>
#include <compat/twi.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include "init.h"
#include "main.h"



// **********************************************************************************
// *** Output handler
//  varifies that the output is ready and flips its status
//  (bit = OUT# to change, enable = 1 for on, 0 for off, 2 for flip)


void output(uint8_t* bit, uint8_t enable){

	//extern uint8_t outputs;
	if (enable == ON)							// if we want to set the bit
	{
		if ( (PORTD & (1 << (*bit+1))) == 0)	//	  if the bit is not set
		{
			outputs |= (1<<(*bit-1)); 			//    turn the bit on
		}
	}
	else if (enable == OFF)						// if we want to reset the bit
	{	
		if ( (PORTD & (1<<(*bit+1))) > 0)		//    if the bit is set
		{
			outputs &= ~(1<<(*bit-1)); 			//    turn the bit off
		}
	}
	else if (enable == FLIP)
	{
		outputs ^= (1<<(*bit-1));
	}
	// else do nothing if enable is off
}



// **********************************************************************************
// *** main routine

int main (void){

// *** SETUP GLOBAL INTERUPTS
	sei(); 								// enable global interrupts


// *** INITULIZE ALL FUNCTIONS
	IOSetupInit();						// SETUP IO
	TimerInit();						// SETUP TIMER
	UartInit();							// SETUP USART 


// *** VARIABLES

	uint8_t command = 0;
	uint16_t cmd_addr = 0;
	uint8_t cmd_value = 0;
	uint8_t cmd_step = 0;

// *** MAIN LOOP
	while (1)
	{

// *** UART - Respond to commands captured by the interrupt
		if (ReceivedByte != 0)
		{

			// from A to Z + [ / ]
			if (ReceivedByte > 64 && ReceivedByte < 94)
			{
				uart_macros();
				ReceivedByte = 0;
			}
			// from a to z
			else if (ReceivedByte > 96 && ReceivedByte < 123 && cmd_step == 0)
			{
				command = command + (ReceivedByte - 96);
				ReceivedByte = 0;
			}

			// terminated if a space
			else if (ReceivedByte == 13)
			{
				cmd_step++;
				ReceivedByte = 0;
			}
			else if (cmd_step > 0)
			{
				if (cmd_step == 1)
				{
				/*
					tmp = 
					cmd_addr = cmd_addr + tmp;
				
				*/
				}
			
				else if (cmd_step > 2)
				{

				}
			}
			
			// terminates if enter is hit
			if (ReceivedByte == 32)
			{
				
				uart_commands(&command /*&cmd_addr. &cmd_value */);
				command = 0;
				cmd_addr = 0;
				cmd_value = 0;
				cmd_step == 0;
				ReceivedByte = 0;
			}

		
		}

		// *** FORCE MASK OUTPUTS (FROM USART)
		PORTD = ((outputs | output_force_mask) << 2);

	}
// *** END MAIN LOOP

} 


// **********************************************************************************
// *** Interupts


// *** TIMER OVERFLOW COMPLETE 1s
ISR(TIMER1_COMPA_vect){

	// incriment the base timer
	timer++;

	if (timer >= 7)
	{
		timer = 1;
	}
	
	//output(&timer, FLIP);
			
}



// USART RECIVED DATA INTERUPT
ISR(USART_RX_vect){ 
	//extern char ReceivedByte;

	// Capture input data so that we could do something with it in the main loop.
	ReceivedByte = UDR0; 					// Fetch the received byte value into the variable "ByteReceived" 
	
}
